@@ -6,6 +6,7 @@
 #
 
 from django.core.urlresolvers import get_mod_func
+from django.core.exceptions import ImproperlyConfigured
 from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponsePermanentRedirect
 from django.conf import settings
 from django.template import RequestContext
@@ -32,7 +33,7 @@ TEMPLATE_RENDERERS = {}
 def route_request(request):
     '''The main router for all calls coming in to the system.'''
     # output the variables so the programmer can debug where this is routing
-    log.debug('controller :: processing: app=%s, page=%s, funcname=%s, urlparams=%s' % (request.controller_app, request.controller_page, request.controller_funcname, request.urlparams))
+    log.debug('DMP :: processing: app=%s, page=%s, funcname=%s, urlparams=%s' % (request.controller_app, request.controller_page, request.controller_funcname, request.urlparams))
 
     # first try going to the view function for this request
     # we look for a views/name.py file where name is the same name as the HTML file
@@ -48,32 +49,34 @@ def route_request(request):
         if os.path.exists(full_module_filename):
           module_obj = importlib.import_module(request.controller_view_module)
           if hasattr(module_obj, request.controller_view_function):
-            log.debug('controller :: calling view function %s.%s' % (request.controller_view_module, request.controller_view_function))
+            log.debug('DMP :: calling view function %s.%s' % (request.controller_view_module, request.controller_view_function))
             try: 
               response = getattr(module_obj, request.controller_view_function)(request)
               if not isinstance(response, HttpResponse):
-                log.debug('controller :: view function %s.%s failed to return an HttpResponse.  Returning Http404.' % (request.controller_view_module, request.controller_view_function))
+                log.debug('DMP :: view function %s.%s failed to return an HttpResponse.  Returning Http404.' % (request.controller_view_module, request.controller_view_function))
                 raise Http404
             except RedirectException as e: # redirect to another page
-              log.debug('controller :: view function %s.%s redirected processing to %s' % (request.controller_view_module, request.controller_view_function, e.redirect_to))
+              log.debug('DMP :: view function %s.%s redirected processing to %s' % (request.controller_view_module, request.controller_view_function, e.redirect_to))
               if e.permanent:
                 return HttpResponsePermanentRedirect(e.redirect_to)
               return HttpResponseRedirect(e.redirect_to)
           else:
-            log.debug('controller :: view function %s not in module %s; returning 404 not found' % (request.controller_view_function, request.controller_view_module))
+            log.debug('DMP :: view function %s not in module %s; returning 404 not found' % (request.controller_view_function, request.controller_view_module))
             raise Http404
         else:
-          log.debug('controller :: module %s not found; sending processing directly to the template' % (request.controller_view_module))
+          log.debug('DMP :: module %s not found; sending processing directly to the template' % (request.controller_view_module))
         break
       except InternalViewRedirectException as ivr:
         request.controller_view_module = ivr.redirect_module
         request.controller_view_function = ivr.redirect_function
         full_module_filename = os.path.normpath(os.path.join(settings.BASE_DIR, request.controller_view_module.replace('.', '/') + '.py'))
-        log.debug('controller :: received an InternalViewRedirect to %s -> %s' % (full_module_filename, request.controller_view_function))
+        log.debug('DMP :: received an InternalViewRedirect to %s -> %s' % (full_module_filename, request.controller_view_function))
       
     # if we get here, a matching view wasn't found; look for a matching template
     if response == None and request.controller_app in TEMPLATE_RENDERERS:
       response = TEMPLATE_RENDERERS[request.controller_app].render_to_response(request, '%s.html' % request.controller_page)
+    else:
+        log.debug('DMP :: app %s is not a designated DMP app.' % (request.controller_app))
   
     # return the response
     if response == None:
@@ -128,7 +131,8 @@ class MakoTemplateRenderer:
     '''Creates a renderer to the given path (relateive to the project root where settings.STATIC_ROOT points to)'''
     project_path = os.path.normpath(settings.BASE_DIR)
     self.app_path = app_path
-    self.template_search_dirs = [ os.path.abspath(os.path.join(project_path, self.app_path, template_subdir)) ] + settings.MAKO_TEMPLATES_DIRS
+    template_dir = get_app_template_dir(app_path, template_subdir)  # raises ImproperlyConfigured if error
+    self.template_search_dirs = [ template_dir ] + settings.MAKO_TEMPLATES_DIRS
     self.cache_root = os.path.abspath(os.path.join(project_path, app_path, settings.MAKO_TEMPLATES_CACHE_DIR, template_subdir)) 
     self.tlookup = TemplateLookup(directories=self.template_search_dirs, imports=settings.MAKO_DEFAULT_TEMPLATE_IMPORTS, module_directory=self.cache_root, collection_size=2000, filesystem_checks=settings.DEBUG)
 
@@ -160,7 +164,7 @@ class MakoTemplateRenderer:
       template_obj.template_full_path = template_obj.filename
     if not hasattr(template_obj, 'mako_template_renderer'):  # if the first time, add a reference to this renderer object
       template_obj.mako_template_renderer = self
-    log.debug('controller :: rendering template %s' % template_obj.filename)
+    log.debug('DMP :: rendering template %s' % template_obj.filename)
     if settings.DEBUG:
       try:
         return template_obj.render_unicode(**context_dict)
@@ -184,7 +188,7 @@ class MakoTemplateRenderer:
       content = self.render(request, template, params)
       return HttpResponse(content.encode(settings.DEFAULT_CHARSET), content_type='%s; charset=%s' % (content_type, settings.DEFAULT_CHARSET))
     except TopLevelLookupException: # template file not found    
-      log.debug('controller :: template "%s" not found in search path: %s.' % (template, self.template_search_dirs))
+      log.debug('DMP :: template "%s" not found in search path: %s.' % (template, self.template_search_dirs))
       raise Http404()
     except RedirectException as e: # redirect to another page
       if e.permanent:
@@ -193,15 +197,32 @@ class MakoTemplateRenderer:
 
 
 
+def get_app_template_dir(appname, template_subdir="templates"):
+  '''Checks whether an app seems to be a valid Django-Mako-Plus app, then returns its template directory'''
+  try:
+    module_obj = importlib.import_module(appname)
+  except ImportError:
+    raise ImproperlyConfigured('DMP :: Cannot create MakoTemplateRenderer: App %s does not exist.' % appname)
+  try:
+    if not module_obj.DJANGO_MAKO_PLUS:
+      raise ImproperlyConfigured('DMP :: Cannot create MakoTemplateRenderer: %s.DJANGO_MAKO_PLUS must be True.' % appname)
+  except AttributeError:
+    raise ImproperlyConfigured('DMP :: Cannot create MakoTemplateRenderer: App %s must define DJANGO_MAKO_PLUS=True.' % appname)
+  template_dir = os.path.abspath(os.path.join(os.path.dirname(module_obj.__file__), template_subdir))
+  if not os.path.isdir(template_dir):
+    raise ImproperlyConfigured('DMP :: Cannot create MakoTemplateRenderer: App %s has no templates folder (it needs %s).' % (appname, templatedir))
+  return template_dir
+
 
 ##########################################################
 ###   Populate the available template renderers
 
-for dirname in os.listdir(settings.BASE_DIR):
-  dirpath = os.path.abspath(os.path.join(settings.BASE_DIR, dirname))
-  if os.path.isdir(dirpath) and dirname in settings.INSTALLED_APPS and os.path.isdir(os.path.join(dirpath, 'templates')):
-    TEMPLATE_RENDERERS[dirname] = MakoTemplateRenderer(dirname)
-
+for appname in settings.INSTALLED_APPS:
+  try:
+    get_app_template_dir(appname) # just to check it
+    TEMPLATE_RENDERERS[appname] = MakoTemplateRenderer(appname)
+  except ImproperlyConfigured:
+    pass
 
 
 ##########################################################
@@ -225,18 +246,28 @@ class RequestInitMiddleware:
     # split the path
     path_parts = request.path[1:].split('/') # [1:] to remove the leading /
       
-    # get the app
-    if len(path_parts) >= 1 and path_parts[0] == '':  # app specified by empty, so revert to default app
-      path_parts[0] = settings.MAKO_DEFAULT_APP
-    elif len(path_parts) < 1:  # app not specified, or invalid app, so insert the default app into the path_parts
-      path_parts.insert(0, settings.MAKO_DEFAULT_APP)
-    request.controller_app = path_parts[0]
+    # ensure that we have at least 2 path_parts to work with
+    # by adding the default app and/or page as needed
+    if len(path_parts) == 0:
+      path_parts.append(settings.MAKO_DEFAULT_APP)
+      path_parts.append(settings.MAKO_DEFAULT_PAGE)
       
-    # get the page
-    if len(path_parts) < 2:  # page not specified, so insert the default page into the path_parts (we'll validate later in the controller)
-      path_parts.insert(1, settings.MAKO_DEFAULT_PAGE)
-    elif path_parts[1] == '':  # page specified by empty
-      path_parts[1] = settings.MAKO_DEFAULT_PAGE
+    elif len(path_parts) == 1: # /app or /page
+      if path_parts[0] in TEMPLATE_RENDERERS:  # one of our apps specified, so insert the default page
+        path_parts.append(settings.MAKO_DEFAULT_PAGE)
+      else:  # not one of our apps, so insert the app and assume path_parts[0] is a page in that app
+        path_parts.insert(0, settings.MAKO_DEFAULT_APP)
+        if not path_parts[1]: # was the page empty?
+          path_parts[1] = settings.MAKO_DEFAULT_PAGE
+    
+    else: # at this point in the elif, we know len(path_parts) >= 2
+      if path_parts[0] not in TEMPLATE_RENDERERS: # the first part was not one of our apps, so insert the default app
+        path_parts.insert(0, settings.MAKO_DEFAULT_APP)
+      if not path_parts[1]:  # is the page empty?
+        path_parts[1] = settings.MAKO_DEFAULT_PAGE
+        
+    # set the app and page in the request
+    request.controller_app = path_parts[0]
     request.controller_page = path_parts[1]
     
     # see if a function is specified with the page (the __ separates a function name)
